# AUTOGENERATED! DO NOT EDIT! File to edit: 06_preprocess.ipynb (unless otherwise specified).

__all__ = ['fc_count2tpm', 'fc_count2fpkm', 'featurecount_rename']

# Cell

import pandas as pd

# Cell

def fc_count2tpm(df,samples):
    results = df.copy()
    rate=results[samples].divide(results['Length'],axis=0)
    results[samples]=rate/rate.sum(axis=0)*1e6
    return results

def fc_count2fpkm(df,samples):
    results = df.copy()
    total_reads = results[samples].sum(axis=0)
    rate = results[samples].divide(results['Length'],axis=0)
    results[samples] = rate.divide(total_reads,axis=1) * 1e6
    return results



# Cell

def featurecount_rename(featurecounts,clinical,prefix,*,sample_title='sample',bam_title='bam',count_title='Geneid'):
    '''
    featurecounts will use the bamfile name as the sample name. This script rename the featurecounts by the given table.

    :param str featurecounts: featurecounts table file
    :param str clinical: clinical table file with header
    :param str prefix: output prefix, there will be four outputs, one is count output (prefix_count.txt), and the other one is the rename featurecounts (prefix_featurecounts.txt) and FPKM and TPM {prefix}_featurecounts_FPKM.txt {prefix}_featurecounts_TPM.txt
    :param str sample_title: the column name of the sample name
    :param str bam_title: the column name of the bamfile name
    :param str count_title: the column name used as identity in count data
    '''

    fc = pd.read_csv(featurecounts,sep='\t',comment='#')
    cli = pd.read_csv(clinical,comment='#')
    fc.rename(columns=cli.set_index(bam_title)[sample_title],
                 inplace=True)
    samples=list(cli[sample_title])
    fc.to_csv(f'{prefix}_featurecounts.txt',sep='\t',index=False)
    fc[[count_title]+samples].to_csv(f'{prefix}_counts.txt',sep='\t',index=False)
    fpkm=fc_count2fpkm(fc,samples)
    tpm=fc_count2tpm(fc,samples)
    fpkm.to_csv(f'{prefix}_featurecounts_FPKM.txt',sep='\t',index=False)
    tpm.to_csv(f'{prefix}_featurecounts_TPM.txt',sep='\t',index=False)